Option Explicit

' Get the arguments and the argument count.
Dim obArgs
Dim cArgs
Dim packageProgId
Dim packageName

' Set the package name.
packageName = "Exoledb Script Handler"
'set the package prog id
packageProgId = "ExOledb.ScriptEventSink.1"

Set obArgs = WScript.Arguments
cArgs = obArgs.Count

ValidateArguments

Sub ValidateArguments()
    If cArgs < 1 Then
        LogError "usage"
        Exit Sub
    End If
    
    Select Case LCase(obArgs.Item(0))
        Case "install"
			If cArgs < 3 Then
				LogError "usage"
				Exit Sub
			End If
            ProcessRequest
        Case "uninstall"
            ProcessRequest
        Case Else
            LogError "usage"
			Exit Sub
    End Select
End Sub

Sub LogUsage()
	Wscript.Echo ( "Usage: cscript eshmts.vbs <install | uninstall> <username> <password>")
    Wscript.Echo ( "Example :")
    Wscript.Echo ( "cscript eshmts.vbs install myuser mypassword")
    Wscript.Echo ( "cscript eshmts.vbs uninstall")
End Sub

Sub LogError (errString)
	If errString = "usage" Then
		LogUsage 
	Else
		Wscript.Echo errString
		If Err.Number < 0 Then
			Wscript.Echo "Error Number: " & Hex(Err.Number)
		Else
			Wscript.Echo "Error Number: " & Err.Number
		End If
		If Err.Description <> "" Then
			Wscript.Echo "Error Description: " & Err.Description
		End If
	End If
End Sub

Sub ProcessRequest()
    Dim catalog
    Dim packages
    On Error Resume Next
    
    ' First, create the catalog object.
    Set catalog = CreateObject("MTSAdmin.Catalog.1")
    If Err.Number <> 0 Then
        LogError "Failed to create the catalog object."
		Err.Clear
        Exit Sub
    End If

    ' Then get the packages collection.
    Set packages = catalog.GetCollection("Packages")
    If Err.Number <> 0 Then
        LogError "Failed to get the packages collection."
		Err.Clear
        Exit Sub
    End If

    packages.Populate
    If Err.Number <> 0 Then
        LogError "Failed to populate the packages collection."
		Err.Clear
        Exit Sub
    End If

    ' Remove all packages that go by the same name as the package you want to install.
    Remove_Package packages, packageName

    If LCase(obArgs.Item(0)) = "install" Then
        'we need to install new package
        Create_Pkg_Install_Components packages, packageName
	End If
End Sub


Sub Create_Pkg_Install_Components(packages, packageName)
    Dim newpack
    Dim components
    Dim util
    Dim i
    Dim newpackID
    On Error Resume Next

    ' Add a new package.
    Set newpack = packages.Add
    If Err.Number <> 0 Then
        LogError "Failed to add a new package."
		Err.Clear
        Exit Sub
    End If
    
    newpackID = newpack.Key
    If Err.Number <> 0 Then
        LogError "Failed to get the new package key."
		Err.Clear
        Remove_Package packages, packageName
        Exit Sub
    End If
    
    newpack.Value("Name") = packageName
    If Err.Number <> 0 Then
        LogError "Failed to get the new package name."
		Err.Clear
        Remove_Package packages, packageName
        Exit Sub
    End If
    
    ' Run the package as the user name passed in.
    newpack.Value("Identity") = obArgs.Item(1)
    If Err.Number <> 0 Then
        LogError "Failed to set the new package identity."
		Err.Clear
        Remove_Package packages, packageName
        Exit Sub
    End If
    
    ' Set the password passed in.
    newpack.Value("Password") = obArgs.Item(2)
    If Err.Number <> 0 Then
        LogError "Failed to set the new package user password."
		Err.Clear
        Remove_Package packages, packageName
        Exit Sub
    End If
    
    packages.SaveChanges
    If Err.Number <> 0 Then
        LogError "Failed to save changes on packages."
		Err.Clear
        Remove_Package packages, packageName
        Exit Sub
    End If
    
    Set components = packages.GetCollection("ComponentsInPackage", newpackID)
    If Err.Number <> 0 Then
        LogError "Failed to get packages components."
		Err.Clear
        Remove_Package packages, packageName
        Exit Sub
    End If
    
    Set util = components.GetUtilInterface
    If Err.Number <> 0 Then
        LogError "Failed to get util interface on the components."
		Err.Clear
        Remove_Package packages, packageName
        Exit Sub
    End If
    
    util.ImportComponentByName packageProgId
    If Err.Number <> 0 Then
        LogError "Failed to import component " & packageProgId
		Err.Clear
        Remove_Package packages, packageName
        Exit Sub
    End If
    
    ' Refresh components.
    components.Populate
    If Err.Number <> 0 Then
        LogError "Failed to populate components."
		Err.Clear
        Remove_Package packages, packageName
        Exit Sub
    End If
    
    ' Iterate through the components and change the transaction attributes.
    numComponents = components.Count
    
    For i = numComponents - 1 To 0 Step -1
        Set component = components.Item(i)
        component.Value("Transaction") = "Not Supported"
    Next
    
    ' Commit the changes to the components.
    components.SaveChanges
End Sub

Sub Remove_Package(packages, packageName)
    Dim numPackages
    Dim i, j
    Dim fPackageRemoved
    Dim newPackage
    Dim components, component
    Dim numComponents
	Dim progName
    On Error Resume Next
    
    ' Remove all packages that go by the same name or contain the same component
    ' as the package you want to install.
    numPackages = packages.Count
    For i = numPackages - 1 To 0 Step -1
        fPackageRemoved = False
        
        ' Check the package name first.
        If (packages.Item(i).Value("Name") = packageName) Then
            packages.Remove (i)
			If Err.Number <> 0 Then
				LogError "Failed to remove package " & packageName
				Err.Clear
		        Exit Sub
			End If
            fPackageRemoved = True
        End If
        
        ' Check the component prog id if the package is not already removed.
        If fPackageRemoved = False Then
            Set newPackage = packages.Item(i)
            Set components = packages.GetCollection("ComponentsInPackage", newPackage.Value("ID"))
            ' Enumerate and check each component in the collection
            components.Populate
            numComponents = components.Count
            For j = numComponents - 1 To 0 Step -1
                Set component = components.Item(j)
                progName = component.Value("ProgID")
                ' Component with the same prog id ? Remove the package that
                ' contains the component.
                If progName = packageProgId Then
                    packages.Remove (i)
					If Err.Number <> 0 Then
						LogError "Failed to remove package " & packageName
						Err.Clear
						Exit Sub
					End If
                    fPackageRemoved = True
                    Exit For
                End If
            Next
        End If
    Next

    ' Commit your deletions.
    packages.SaveChanges

End Sub
